// ==UserScript==
// @name      etherpad_mediafragments_xpub
// @namespace http://activearchives.org/gmscripts
// @include   https://pad.xpub.nl/p/*
// @version   1
// @grant     none
// ==/UserScript==

console.log("etherpad_mediafragments...");

function create_media (href, mediatagname="audio") {
    let chat = document.querySelector("#chatbox .chat-content");
  let div = document.createElement("div");
  let idiv = document.createElement("div");
  let audio = document.createElement(mediatagname);
  audio.style = "width: 100%";
  audio.controls = true;
  audio.src = href;
  idiv.appendChild(audio);
  div.appendChild(idiv);
  chat.appendChild(div);
  audio.addEventListener("play", function (e) {
      current_media = audio;
  });
  return audio;
}

let current_media = null;
const media_by_url = {};


function parse_hash (hash) {
  // console.log("hash", hash);
  let m = hash.match(/#t=((\d\d):)?(\d\d):(\d\d(\.\d\d\d)?)\b/);
  if (m) {
    // console.log("match", m);
    let secs;
    if (m[2]) {
      // console.log("parsing HH:MM:SS");
        secs = (parseInt(m[2]) * 3600) + (parseInt(m[3]) * 60) + parseFloat(m[4])
    } else {
      // console.log("parsing MM:SS");
        secs = (parseInt(m[3]) * 60) + parseFloat(m[4])
    }
        // console.log(`parse_hash ${hash}: ${secs}`);
    return secs;
  }
    return null;
}

function makeTimecode (secs) {
    let original_secs = secs;
    let h = Math.floor(secs / 3600);
    secs -= (h * 3600);
    let m = Math.floor(secs/60);
    secs -= (m * 60);
      let s = Math.floor(secs);
    secs -= s;
    let ret;
        if (h) {
      ret = `${h.toString().padStart(2, '0')}:${m.toString().padStart(2, '0')}:${s.toString().padStart(2, '0')}`;
        } else {
      ret =`${m.toString().padStart(2, '0')}:${s.toString().padStart(2, '0')}`;
        }
    // remaining secs is fractional part 0 < secs < 1
    if (secs) {
      ret += `.${secs.toFixed(3).slice(2)}`;
    }
    // console.log(`TC: ${original_secs} -> ${ret}`);
    return ret;
}

function media_click (baseurl, hash, mediatagname="audio") {
  let media = media_by_url[baseurl];
    if (!media) {
    media = create_media(baseurl, mediatagname);
    media_by_url[baseurl] = media;
  }
  if (hash) {
    let t = parse_hash(hash);
    if (t !== null) {
      media.currentTime = t;
    }
  }
  media.play();
  current_media = media;
}

// adapted from https://developer.mozilla.org/en-US/docs/Web/API/Element/paste_event
function custom_paste (paste) {
  const selection = inner_win.getSelection();
  if (!selection.rangeCount) return;
  selection.deleteFromDocument();
  selection.getRangeAt(0).insertNode(document.createTextNode(paste));
  selection.collapseToEnd();
}

function copy (text) {
}

function jump (time) {
  if (current_media) {
    current_media.currentTime += time;
  }
}

function togglePlayback () {
  if (current_media) {
    if (current_media.paused) {
      current_media.play();
    } else {
      current_media.pause();
    }
  }
}

function pasteTimecode() {
    if (current_media) {
    let t = current_media.currentTime;
    let tc = makeTimecode(t);
    let link = `${current_media.src}#t=${tc}`;
    console.log(link);
    navigator.clipboard.writeText(link).catch(function (e) {
      console.log("clipboard.writeText, error:", e);
    });
    custom_paste(link);
  }
}

let inner_win;

function watch (frame) {
    let doc = frame.contentDocument;
  let win = frame.contentWindow;
    inner_win = win;

  // Patch Clicks to Media Links
  doc.addEventListener("click", function (e) {
    // console.log("click", e);
    if (e.target.nodeName == "A") {
      let url = new URL(e.target.href);
      let hash = url.hash;
      // console.log("clicked url", url);
      url.hash = "";
      let baseurl = `${url}`;
      let ext = baseurl.match(/\/(listen)$|\.(\w+)$/);
      // add support for radio WORM's live stream
      if (ext) {
        ext = ext[1].toLowerCase();
        if (/^mp3|ogg|(listen)|m4a$/i.test(ext)) {
          // console.log(`audio_click: baseurl: ${baseurl}, hash: ${hash}`);
          // console.log(`!${baseurl}, ${ext}`);
          // console.log("attempting to cancel default action");
          e.preventDefault();
          e.stopPropagation();
          media_click(baseurl, hash, "audio");
        } else if (/^mp4|webm|m4v$/i.test(ext)) {
          e.preventDefault();
          e.stopPropagation();
          media_click(baseurl, hash, "video");
        }
      }
    }
  }, true);

  // Add Keyboard shortcuts
  doc.addEventListener("keydown", function (e) {
    // console.log("keydown", e);
    if ((e.ctrlKey || e.metaKey) && e.shiftKey) {
      if (e.key == "ArrowDown") {
        pasteTimecode();
        e.preventDefault(); e.stopPropagation();
      } else if (e.key == "ArrowLeft") {
        jump(-5);
        e.preventDefault(); e.stopPropagation();
      } else if (e.key == "ArrowRight") {
            jump(5);
        e.preventDefault(); e.stopPropagation();
      } else if (e.key == "ArrowUp") {
        togglePlayback();
        e.preventDefault(); e.stopPropagation();
      }
    }
  }, true);


  console.log("media fragment extension running...");
}

function init () {
  let ace_outer = document.querySelector("iframe[name=ace_outer]");
  // console.log("ace_outer", ace_outer);
  if (ace_outer) {
    let ace_inner = ace_outer.contentDocument.querySelector("iframe[name=ace_inner]");
    // console.log("ace_inner", ace_inner);
    if (ace_inner) {
      watch(ace_inner);
      return true;
    }
  }
}

const waitFor = delay => new Promise(resolve => setTimeout(resolve, delay));

async function try_to_init () {
  for (var i=0; i<100; i++) {
      let ok = init();
    if (ok) {
      console.log("etherpad_mediafragments... initialized");
      return true;
    }
    await waitFor(250);
  }
}

window.addEventListener("DOMContentLoaded", function (e) {
    try_to_init();
});
